#ifndef CODONDEF_H
#define CODONDEF_H

#include "types.h"
#include "alphadef.h"

#define CODONLENGTH              3
#define NOTRANSLATIONSCHEME      0
#define MAXTRANSLATIONTABLE      23
#define STANDARDTRANSLATIONTABLE UintConst(1)
#define AMINOACIDFAIL            ((char) -1)

#endif
