//\IgnoreLatex{

#ifndef LOG2_H
#define LOG2_H
#include <math.h>

//}

/*
  This file defines some defines for logarithms.
*/

#define LOGof2  0.693147                    // log 2
#define LOGof10 2.30259                     // log 10
#define LOG2(D) log((double) (D))/LOGof2    // log2 d

//\IgnoreLatex{

#endif

//}
