use strict;
use warnings;

sub makemkvtreeargumenttable
{
  my($DBdna,$DBprot) = @_;
  my @arglisttable =
  (
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois  -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis -ois  ",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis   -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis   -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1 -tis   ",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois  -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1  -ois  ",
   " -dna -db $DBdna -bck -sti1 -pl 1   -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1   -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1    -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1    -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 1    -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 1    ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois  -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis -ois  ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis   -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis   -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2 -tis   ",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois  -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2  -ois  ",
   " -dna -db $DBdna -bck -sti1 -pl 2   -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2   -skp -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2    -suf -lcp -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2    -suf -lcp ",
   " -dna -db $DBdna -bck -sti1 -pl 2    -bwt",
   " -dna -db $DBdna -bck -sti1 -pl 2    ",
   " -dna -db $DBdna -bck -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1 -tis -ois  -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis -ois  ",
   " -dna -db $DBdna -bck -pl 1 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis   -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1 -tis   -bwt",
   " -dna -db $DBdna -bck -pl 1 -tis   ",
   " -dna -db $DBdna -bck -pl 1  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1  -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1  -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1  -ois  -bwt",
   " -dna -db $DBdna -bck -pl 1  -ois  ",
   " -dna -db $DBdna -bck -pl 1   -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1   -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1    -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 1    -suf -lcp ",
   " -dna -db $DBdna -bck -pl 1    -bwt",
   " -dna -db $DBdna -bck -pl 1    ",
   " -dna -db $DBdna -bck -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2 -tis -ois  -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis -ois  ",
   " -dna -db $DBdna -bck -pl 2 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis   -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2 -tis   -bwt",
   " -dna -db $DBdna -bck -pl 2 -tis   ",
   " -dna -db $DBdna -bck -pl 2  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2  -ois -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2  -ois  -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2  -ois  -bwt",
   " -dna -db $DBdna -bck -pl 2  -ois  ",
   " -dna -db $DBdna -bck -pl 2   -skp -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2   -skp -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2    -suf -lcp -bwt",
   " -dna -db $DBdna -bck -pl 2    -suf -lcp ",
   " -dna -db $DBdna -bck -pl 2    -bwt",
   " -dna -db $DBdna -bck -pl 2    ",
   " -dna -db $DBdna  -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 1 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna  -pl 1 -tis -ois  -bwt",
   " -dna -db $DBdna  -pl 1 -tis -ois  ",
   " -dna -db $DBdna  -pl 1 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 1 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1 -tis   -suf -lcp ",
   " -dna -db $DBdna  -pl 1 -tis   -bwt",
   " -dna -db $DBdna  -pl 1 -tis   ",
   " -dna -db $DBdna  -pl 1  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1  -ois -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 1  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1  -ois  -suf -lcp ",
   " -dna -db $DBdna  -pl 1  -ois  -bwt",
   " -dna -db $DBdna  -pl 1  -ois  ",
   " -dna -db $DBdna  -pl 1   -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1   -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 1    -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 1    -suf -lcp ",
   " -dna -db $DBdna  -pl 1    -bwt",
   " -dna -db $DBdna  -pl 1    ",
   " -dna -db $DBdna  -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2 -tis -ois -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 2 -tis -ois  -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2 -tis -ois  -suf -lcp ",
   " -dna -db $DBdna  -pl 2 -tis -ois  -bwt",
   " -dna -db $DBdna  -pl 2 -tis -ois  ",
   " -dna -db $DBdna  -pl 2 -tis  -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2 -tis  -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 2 -tis   -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2 -tis   -suf -lcp ",
   " -dna -db $DBdna  -pl 2 -tis   -bwt",
   " -dna -db $DBdna  -pl 2 -tis   ",
   " -dna -db $DBdna  -pl 2  -ois -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2  -ois -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 2  -ois  -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2  -ois  -suf -lcp ",
   " -dna -db $DBdna  -pl 2  -ois  -bwt",
   " -dna -db $DBdna  -pl 2  -ois  ",
   " -dna -db $DBdna  -pl 2   -skp -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2   -skp -suf -lcp ",
   " -dna -db $DBdna  -pl 2    -suf -lcp -bwt",
   " -dna -db $DBdna  -pl 2    -suf -lcp ",
   " -dna -db $DBdna  -pl 2    -bwt",
   " -dna -db $DBdna  -pl 2    ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis -ois  ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis   -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1 -tis   ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1  -ois  ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1    -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1    -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 1    ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis -ois  ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis   -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2 -tis   ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2  -ois  ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2    -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2    -bwt",
   " -smap TransDNA -db $DBdna -bck -sti1 -pl 2    ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis -ois  ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis   -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1 -tis   ",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1  -ois  ",
   " -smap TransDNA -db $DBdna -bck -pl 1   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1    -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 1    -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 1    ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis -ois  ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis   -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2 -tis   ",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois  -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2  -ois  ",
   " -smap TransDNA -db $DBdna -bck -pl 2   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2    -suf -lcp ",
   " -smap TransDNA -db $DBdna -bck -pl 2    -bwt",
   " -smap TransDNA -db $DBdna -bck -pl 2    ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis -ois  ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1 -tis   -bwt",
   " -smap TransDNA -db $DBdna  -pl 1 -tis   ",
   " -smap TransDNA -db $DBdna  -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1  -ois  -bwt",
   " -smap TransDNA -db $DBdna  -pl 1  -ois  ",
   " -smap TransDNA -db $DBdna  -pl 1   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 1    -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 1    -bwt",
   " -smap TransDNA -db $DBdna  -pl 1    ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois  -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis -ois  ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis   -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2 -tis   -bwt",
   " -smap TransDNA -db $DBdna  -pl 2 -tis   ",
   " -smap TransDNA -db $DBdna  -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2  -ois -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2  -ois  -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2  -ois  -bwt",
   " -smap TransDNA -db $DBdna  -pl 2  -ois  ",
   " -smap TransDNA -db $DBdna  -pl 2   -skp -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2   -skp -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2    -suf -lcp -bwt",
   " -smap TransDNA -db $DBdna  -pl 2    -suf -lcp ",
   " -smap TransDNA -db $DBdna  -pl 2    -bwt",
   " -smap TransDNA -db $DBdna  -pl 2    ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois  -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis -ois  ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis   -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis   -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1 -tis   ",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois  -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1  -ois  ",
   " -protein -db $DBprot -bck -sti1 -pl 1   -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1   -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1    -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1    -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 1    -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 1    ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois  -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis -ois  ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis   -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis   -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2 -tis   ",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois  -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2  -ois  ",
   " -protein -db $DBprot -bck -sti1 -pl 2   -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2   -skp -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2    -suf -lcp -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2    -suf -lcp ",
   " -protein -db $DBprot -bck -sti1 -pl 2    -bwt",
   " -protein -db $DBprot -bck -sti1 -pl 2    ",
   " -protein -db $DBprot -bck -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1 -tis -ois  -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis -ois  ",
   " -protein -db $DBprot -bck -pl 1 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis   -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1 -tis   -bwt",
   " -protein -db $DBprot -bck -pl 1 -tis   ",
   " -protein -db $DBprot -bck -pl 1  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1  -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1  -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1  -ois  -bwt",
   " -protein -db $DBprot -bck -pl 1  -ois  ",
   " -protein -db $DBprot -bck -pl 1   -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1   -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1    -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 1    -suf -lcp ",
   " -protein -db $DBprot -bck -pl 1    -bwt",
   " -protein -db $DBprot -bck -pl 1    ",
   " -protein -db $DBprot -bck -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2 -tis -ois  -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis -ois  ",
   " -protein -db $DBprot -bck -pl 2 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis   -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2 -tis   -bwt",
   " -protein -db $DBprot -bck -pl 2 -tis   ",
   " -protein -db $DBprot -bck -pl 2  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2  -ois -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2  -ois  -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2  -ois  -bwt",
   " -protein -db $DBprot -bck -pl 2  -ois  ",
   " -protein -db $DBprot -bck -pl 2   -skp -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2   -skp -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2    -suf -lcp -bwt",
   " -protein -db $DBprot -bck -pl 2    -suf -lcp ",
   " -protein -db $DBprot -bck -pl 2    -bwt",
   " -protein -db $DBprot -bck -pl 2    ",
   " -protein -db $DBprot  -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 1 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot  -pl 1 -tis -ois  -bwt",
   " -protein -db $DBprot  -pl 1 -tis -ois  ",
   " -protein -db $DBprot  -pl 1 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 1 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1 -tis   -suf -lcp ",
   " -protein -db $DBprot  -pl 1 -tis   -bwt",
   " -protein -db $DBprot  -pl 1 -tis   ",
   " -protein -db $DBprot  -pl 1  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1  -ois -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 1  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1  -ois  -suf -lcp ",
   " -protein -db $DBprot  -pl 1  -ois  -bwt",
   " -protein -db $DBprot  -pl 1  -ois  ",
   " -protein -db $DBprot  -pl 1   -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1   -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 1    -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 1    -suf -lcp ",
   " -protein -db $DBprot  -pl 1    -bwt",
   " -protein -db $DBprot  -pl 1    ",
   " -protein -db $DBprot  -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2 -tis -ois -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 2 -tis -ois  -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2 -tis -ois  -suf -lcp ",
   " -protein -db $DBprot  -pl 2 -tis -ois  -bwt",
   " -protein -db $DBprot  -pl 2 -tis -ois  ",
   " -protein -db $DBprot  -pl 2 -tis  -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2 -tis  -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 2 -tis   -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2 -tis   -suf -lcp ",
   " -protein -db $DBprot  -pl 2 -tis   -bwt",
   " -protein -db $DBprot  -pl 2 -tis   ",
   " -protein -db $DBprot  -pl 2  -ois -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2  -ois -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 2  -ois  -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2  -ois  -suf -lcp ",
   " -protein -db $DBprot  -pl 2  -ois  -bwt",
   " -protein -db $DBprot  -pl 2  -ois  ",
   " -protein -db $DBprot  -pl 2   -skp -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2   -skp -suf -lcp ",
   " -protein -db $DBprot  -pl 2    -suf -lcp -bwt",
   " -protein -db $DBprot  -pl 2    -suf -lcp ",
   " -protein -db $DBprot  -pl 2    -bwt",
   " -protein -db $DBprot  -pl 2    ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis -ois  ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis   -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1 -tis   ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1  -ois  ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1    -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1    -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 1    ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis -ois  ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis   -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2 -tis   ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2  -ois  ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2    -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2    -bwt",
   " -smap TransProt11 -db $DBprot -bck -sti1 -pl 2    ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis -ois  ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis   -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1 -tis   ",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1  -ois  ",
   " -smap TransProt11 -db $DBprot -bck -pl 1   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1    -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 1    -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 1    ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis -ois  ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis   -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2 -tis   ",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois  -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2  -ois  ",
   " -smap TransProt11 -db $DBprot -bck -pl 2   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2    -suf -lcp ",
   " -smap TransProt11 -db $DBprot -bck -pl 2    -bwt",
   " -smap TransProt11 -db $DBprot -bck -pl 2    ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis -ois  ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis   -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1 -tis   ",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois  -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1  -ois  ",
   " -smap TransProt11 -db $DBprot  -pl 1   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1    -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 1    -bwt",
   " -smap TransProt11 -db $DBprot  -pl 1    ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois  -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis -ois  ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis  -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis  -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis   -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis   -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis   -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2 -tis   ",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois  -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois  -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois  -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2  -ois  ",
   " -smap TransProt11 -db $DBprot  -pl 2   -skp -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2   -skp -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2    -suf -lcp -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2    -suf -lcp ",
   " -smap TransProt11 -db $DBprot  -pl 2    -bwt",
   " -smap TransProt11 -db $DBprot  -pl 2    ",
  );
  return \@arglisttable;
}

1;
