
/* This file is generated. Do not edit. */
#ifndef ALPHASIZE_H
#define ALPHASIZE_H
#if (ALPHABETSIZE==4)
#define VMATMAXOUT vmatmaxout4
#define VMATMAXCOUNT vmatmaxcount4
#endif
#if (ALPHABETSIZE==12)
#define VMATMAXOUT vmatmaxout12
#define VMATMAXCOUNT vmatmaxcount12
#endif
#if (ALPHABETSIZE==21)
#define VMATMAXOUT vmatmaxout21
#define VMATMAXCOUNT vmatmaxcount21
#endif
#if (ALPHABETSIZE==31)
#define VMATMAXOUT vmatmaxout31
#define VMATMAXCOUNT vmatmaxcount31
#endif
#if (ALPHABETSIZE==42)
#define VMATMAXOUT vmatmaxout42
#define VMATMAXCOUNT vmatmaxcount42
#endif
#if (ALPHABETSIZE==53)
#define VMATMAXOUT vmatmaxout53
#define VMATMAXCOUNT vmatmaxcount53
#endif
#if (ALPHABETSIZE==66)
#define VMATMAXOUT vmatmaxout66
#define VMATMAXCOUNT vmatmaxcount66
#endif
#endif
